//+------------------------------------------------------------------+ 
//|                                    JS_CC_Trend_MultiStack.mq4    | 
//|                                                                  |
//|   Uses Mighty One's TrendSetter idea as its basis.  Once the     |
//|         custom candles are formed, they are "stacked."  This     |
//|         works in a similar fashion to Point & Figure charts.     |
//|         Consecutive up candles are stacked into one candle       |
//|         until there is a down candle.  Then consecutive down     |
//|         candles are stacked in the same manner, etc.             |
//|                                                                  |
//|   This variant allows the user to present stack charts for       |
//|         any of the 4 TS levels.                                  |
//+------------------------------------------------------------------+ 

#property copyright "J Scarlett"

#property indicator_chart_window

extern int CandleBars = 2;
extern int LookBack = 1;
extern bool DoShow1 = true;
extern bool DoShow2 = true;
extern bool DoShow3 = true;
extern bool DoShow4 = false;
extern bool DoFill1 = true;
extern bool DoFill2 = true;
extern bool DoFill3 = false;
extern bool DoFill4 = false;
extern int CandleLineWidth = 1;
extern int CandleLineStyle = 2;
extern int CCBarWidth1 = 2;
extern int CCBarWidth2 = 3;
extern int CCBarWidth3 = 5;
extern int CCBarWidth4 = 6;
extern color CandleColor1 = Black;
extern color CandleColor2 = Silver;
extern color CandleColor3 = Brown;
extern color CandleColor4 = Green;

// ----globals
bool IsUp[4];
datetime LastBar;
int TotLevs = 4;
int CountUp[4] = {0, 0, 0, 0};
int CountDown[4] = {0, 0, 0, 0};
int CustomBars[4] = {0, 0, 0, 0};
int StackedBars[4] = {0, 0, 0, 0};
int LastCC[4] = {0, 0, 0, 0};
int Anchor[4] = {0, 0, 0, 0};

int digits;
double points;

double OpenCC[4], CloseCC[4];
double StackOpen[4], StackClose[4];

bool ShowIt[4], FillIt[4];
int LineWidth, LineStyle, CCBars[4];
color CandleClr[4];

string TAG = "JS_TS_MultiStack ";
string TAG1[4] = {"1 ", "2 ", "3 ", "4 "};
string TAG2[4] = {"Raw1 ", "Raw2 ", "Raw3 ", "Raw4 "};

//----------------------------------------------------------------------------

int init() 
{
   int i;
   
   digits = Digits;
   points = Point; 
   
   deinit();

   if(digits == 5 || digits == 3)
   {
      digits--;
      points *= 10;
   }
   
   ShowIt[0] = DoShow1;
   ShowIt[1] = DoShow2;
   ShowIt[2] = DoShow3;
   ShowIt[3] = DoShow4;
   FillIt[0] = DoFill1;
   FillIt[1] = DoFill2;
   FillIt[2] = DoFill3;
   FillIt[3] = DoFill4;
   LineWidth = CandleLineWidth;
   LineStyle = CandleLineStyle;
   CandleClr[0] = CandleColor1;
   CandleClr[1] = CandleColor2;
   CandleClr[2] = CandleColor3;
   CandleClr[3] = CandleColor4;
   CCBars[0] = CCBarWidth1;
   CCBars[1] = CCBarWidth2;
   CCBars[2] = CCBarWidth3;
   CCBars[3] = CCBarWidth4;
   for (i=0; i<4; i++)
      if (!ShowIt[i])
         CandleClr[i] = CLR_NONE;
   
   LastBar = Time[Bars-1];
   OpenCC[0] = Close[Bars-1];
   CloseCC[0] = Close[Bars-1];
   OpenCC[1] = Close[Bars-1];
   CloseCC[1] = Close[Bars-1];
   OpenCC[2] = Close[Bars-1];
   CloseCC[2] = Close[Bars-1];
   OpenCC[3] = Close[Bars-1];
   CloseCC[3] = Close[Bars-1];
   
   return(0);
      
} // init()

//----------------------------------------------------------------------------

void ObDeleteObjectsByPrefix(string Prefix)
{
   int L = StringLen(Prefix);
   int i = 0; 
   while(i < ObjectsTotal())
     {
       string ObjName = ObjectName(i);
       if(StringSubstr(ObjName, 0, L) != Prefix) 
         { 
           i++; 
           continue;
         }
       ObjectDelete(ObjName);
     }
}
    
//----------------------------------------------------------------------------
int deinit()
{
       
   ObDeleteObjectsByPrefix(TAG);  
   
   return(0);
   
} // deinit();

//----------------------------------------------------------------------------

int start() 
{
   int limit;
   int i;
   string n;

   limit = iBarShift(Symbol(), 0, LastBar) - 1;

   for (i=limit; i>0; i--)
   {
      CheckCC(i, 0);
      
   } // cycle through all bars 

   return(0);
  
} // start()

//----------------------------------------------------------------------------

void DoPaintRaw(string thetag, int shift, int lev, double theopen, double theclose)
{
   ObjectCreate(thetag, OBJ_RECTANGLE, 0, 0, 0);
   ObjectSet(thetag, OBJPROP_WIDTH, LineWidth);
   ObjectSet(thetag, OBJPROP_STYLE, LineStyle);
   ObjectSet(thetag, OBJPROP_TIME1, Time[shift+CCBars[lev]-1]);
   ObjectSet(thetag, OBJPROP_PRICE1, theopen);
   ObjectSet(thetag, OBJPROP_TIME2, Time[shift]);
   ObjectSet(thetag, OBJPROP_PRICE2, theclose);
   ObjectSet(thetag, OBJPROP_COLOR, CLR_NONE);
   
} // DoPaint()

//----------------------------------------------------------------------------

void CheckCC(int shift, int lev)
{
   int j, k, temp, temp2;
   string n;
   
   if (lev == TotLevs)
      return;
      
   if (((lev == 0) && (Close[shift] > Close[shift+LookBack])) || (CloseCC[lev-1] > OpenCC[lev-1]))
   {
      if (CountUp[lev] == (CandleBars - 1))
      {
         CustomBars[lev]++;
         n = TAG + TAG2[lev] + CustomBars[lev];
         OpenCC[lev] = CloseCC[lev];
         if (lev == 0)
            CloseCC[lev] = Close[shift];
         else
            CloseCC[lev] = CloseCC[lev-1];

         DoPaintRaw(n, shift, lev, OpenCC[lev], CloseCC[lev]);

         if (lev > 0)
            Anchor[lev] = CustomBars[lev-1];
            
         CheckCC(shift, lev+1);
      } // completed up CC
      
      CountUp[lev]++;
      CountUp[lev] %= CandleBars;
      CountDown[lev] = 0;
      if (lev == 0)
         LastBar = Time[shift];
      
   } // higher bar or CC
   else
      if (((lev == 0) && (Close[shift] < Close[shift+LookBack])) || (CloseCC[lev-1] < OpenCC[lev-1]))
      {
         if (CountDown[lev] == (CandleBars - 1))
         {
            CustomBars[lev]++;
            n = TAG + TAG2[lev] + CustomBars[lev];
            OpenCC[lev] = CloseCC[lev];
            if (lev == 0)
               CloseCC[lev] = Close[shift];
            else
               CloseCC[lev] = CloseCC[lev-1];

            DoPaintRaw(n, shift, lev, OpenCC[lev], CloseCC[lev]);

            if (lev > 0)
               Anchor[lev] = CustomBars[lev-1];
            
            CheckCC(shift, lev+1);
         } // completed down CC
         
         CountDown[lev]++;
         CountDown[lev] %= CandleBars;
         CountUp[lev] = 0;
         if (lev == 0)
            LastBar = Time[shift];
      } // lower bar or CC
      else
      {
         CountUp[lev] = 0;
         CountDown[lev] = 0;
         if (lev==0)
            LastBar = Time[shift];
      } // doji bar or CC
      
  
   if ((lev == 0) && (shift == 1))
   {
      DoStack();
      
      for (k=0; k<TotLevs; k++)
      {
         for (j=StackedBars[k]; j>0; j--)
         {
            temp = CCBars[k] * (StackedBars[k] - j) + 1;
            n = TAG + TAG1[k] + j;
            ObjectSet(n, OBJPROP_TIME1, Time[temp+CCBars[k]-1]);
            ObjectSet(n, OBJPROP_TIME2, Time[temp]);
         } // relocate stacked CCs
      } // cycle through all levels
   } // only do this when all history is done 

      
} // CheckCC()

//----------------------------------------------------------------------------

void DoStack()
{
   int m;
   double temp;
   string n1, n2;
   
   for (m=0; m<TotLevs; m++)
   {
      if ((StackedBars[m] == 0) && ShowIt[m])
      {
         n1 = TAG + TAG2[m] + 1;
         n2 = TAG + TAG1[m] + 1;
         ObjectCreate(n2, OBJ_RECTANGLE, 0, 0, 0);
         ObjectSet(n2, OBJPROP_BACK, FillIt[m]);
         ObjectSet(n2, OBJPROP_WIDTH, LineWidth);
         ObjectSet(n2, OBJPROP_STYLE, LineStyle);
         ObjectSet(n2, OBJPROP_TIME1, ObjectGet(n1, OBJPROP_TIME1));
         ObjectSet(n2, OBJPROP_TIME2, ObjectGet(n1, OBJPROP_TIME2));
         ObjectSet(n2, OBJPROP_COLOR, CandleClr[m]);
      
         StackOpen[m] = ObjectGet(n1, OBJPROP_PRICE1);
         StackClose[m] = ObjectGet(n1, OBJPROP_PRICE2);
         if (StackClose[m] >= StackOpen[m])
            IsUp[m] = true;
         else
            IsUp[m] = false;
         
         ObjectSet(n2, OBJPROP_PRICE1, StackOpen[m]);
         ObjectSet(n2, OBJPROP_PRICE2, StackClose[m]);
      
         StackedBars[m]++;
         LastCC[m]++;
      } // no stacked bars created yet

      while ((LastCC[m] < CustomBars[m]) && ShowIt[m])
      {
         LastCC[m]++;
         if (IsUp[m])
         {
            n1 = TAG + TAG2[m] + LastCC[m];
            temp = ObjectGet(n1, OBJPROP_PRICE2);
            if (temp >= StackClose[m])
            {
               StackClose[m] = temp;
               n2 = TAG + TAG1[m] + StackedBars[m];
               ObjectSet(n2, OBJPROP_PRICE2, StackClose[m]);
            } // continue up stack CC
            else
            {
               StackedBars[m]++;
               IsUp[m] = false;
               n2 = TAG + TAG1[m] + StackedBars[m];
            
               StackOpen[m] = ObjectGet(n1, OBJPROP_PRICE1);
               StackClose[m] = temp;

               ObjectCreate(n2, OBJ_RECTANGLE, 0, 0, 0);
               ObjectSet(n2, OBJPROP_BACK, FillIt[m]);
               ObjectSet(n2, OBJPROP_WIDTH, LineWidth);
               ObjectSet(n2, OBJPROP_STYLE, LineStyle);
               ObjectSet(n2, OBJPROP_TIME1, ObjectGet(n1, OBJPROP_TIME1));
               ObjectSet(n2, OBJPROP_TIME2, ObjectGet(n1, OBJPROP_TIME2));
               ObjectSet(n2, OBJPROP_PRICE1, StackOpen[m]);
               ObjectSet(n2, OBJPROP_PRICE2, StackClose[m]);
               ObjectSet(n2, OBJPROP_COLOR, CandleClr[m]);
            } // begin a down stack CC
         
         } // current stacked CC is up
         else
         {
            n1 = TAG + TAG2[m] + LastCC[m];
            temp = ObjectGet(n1, OBJPROP_PRICE2);
            if (temp <= StackClose[m])
            {
               StackClose[m] = temp;
               n2 = TAG + TAG1[m] + StackedBars[m];
               ObjectSet(n2, OBJPROP_PRICE2, StackClose[m]);
            } // continue up stack CC
            else
            {
               StackedBars[m]++;
               IsUp[m] = true;
               n2 = TAG + TAG1[m] + StackedBars[m];
            
               StackOpen[m] = ObjectGet(n1, OBJPROP_PRICE1);
               StackClose[m] = temp;

               ObjectCreate(n2, OBJ_RECTANGLE, 0, 0, 0);
               ObjectSet(n2, OBJPROP_BACK, FillIt[m]);
               ObjectSet(n2, OBJPROP_WIDTH, LineWidth);
               ObjectSet(n2, OBJPROP_STYLE, LineStyle);
               ObjectSet(n2, OBJPROP_TIME1, ObjectGet(n1, OBJPROP_TIME1));
               ObjectSet(n2, OBJPROP_TIME2, ObjectGet(n1, OBJPROP_TIME2));
               ObjectSet(n2, OBJPROP_PRICE1, StackOpen[m]);
               ObjectSet(n2, OBJPROP_PRICE2, StackClose[m]);
               ObjectSet(n2, OBJPROP_COLOR, CandleClr[m]);
            } // begin a down stack CC
         
         } // current stacked CC is down
      } // cycle through any new CC
   } // cycle through all levels   
   return;
   
} // DoStack()

//----------------------------------------------------------------------------


