//+------------------------------------------------------------------+ 
//|                                    JS_CC_Trend_Compressed.mq4    | 
//|                              aka "Price Purifier" - MO :)        |
//|                                                                  |
//|   Generates Custom Candles in accordance with the principles     |
//|         laid out by MightyOne in the kreslik.com thread          |
//|         entitled "One Is Not a Trend."                           |
//|                                                                  |
//|   This variant is based on another MO idea to make all of the    |
//|         generated custom candles contiguous.  This essentially   |
//|         shows the price action of multiple time frames on one    |
//|         chart, without the need to scroll back.  It is best      |
//|         viewed without having the underlying bars visible.       |
//|                                                                  |
//|   Version 1.1 (16 Feb 12) - In the original version, the         |
//|         "FixLastCC" option anchored the most recent CC at a      |
//|         given level to the price bar where it formed.  This      |
//|         was visually awkward.  This version continues to anchor  |
//|         the latest level 1 CC to the underlying bar where        |
//|         it formed.  But the latest higher level CCs are now      |
//|         anchored to the lower level CCs that spawned them.       |
//|                                                                  |
//|   Version 1.2 (28 Feb 12) - In the previous version, when the    |
//|         "FixLastCC" option was set to true, setting DoShow       |
//|         for a given level to false also caused higher levels     |
//|         to not be displayed.  This has been corrected.  Also     |
//|         changed the default setting for "FixLastCC" to true.     |
//|                                                                  |
//+------------------------------------------------------------------+ 

#property copyright "J Scarlett"

#property indicator_chart_window

extern int CandleBars = 2;
extern int LookBack = 1;
extern bool DoShow1 = true;
extern bool DoShow2 = true;
extern bool DoShow3 = false;
extern bool DoShow4 = false;
extern bool DoFill1 = false;
extern bool DoFill2 = true;
extern bool DoFill3 = false;
extern bool DoFill4 = true;
extern bool FixLastCC = true;
extern int CandleLineWidth1 = 1;
extern int CandleLineStyle1 = 0;
extern int CandleLineWidth2 = 1;
extern int CandleLineStyle2 = 0;
extern int CandleLineWidth3 = 3;
extern int CandleLineStyle3 = 0;
extern int CandleLineWidth4 = 3;
extern int CandleLineStyle4 = 0;
extern color CandleColor1 = Black;
extern color CandleColor2 = Red;
extern color CandleColor3 = DodgerBlue;
extern color CandleColor4 = Silver;

// ----globals
datetime LastBar;
int TotLevs = 4;
int CountUp[4] = {0, 0, 0, 0};
int CountDown[4] = {0, 0, 0, 0};
int CustomBars[4] = {0, 0, 0, 0};
int Anchor[4] = {0, 0, 0, 0};

int digits;
double points;

double OpenCC[4], CloseCC[4];

bool ShowIt[4], FillIt[4];
int LineWidth[4], LineStyle[4], CCBars[4];
color CandleClr[4];

string TAG = "JS_Price_Purifier";
string TagIt[4] = {" Lev1 ", " Lev2 ", " Lev3 ", " Lev4 "};

//----------------------------------------------------------------------------

int init() 
{
   digits = Digits;
   points = Point; 
   
   deinit();

   if(digits == 5 || digits == 3)
   {
      digits--;
      points *= 10;
   }
   
   ShowIt[0] = DoShow1;
   ShowIt[1] = DoShow2;
   ShowIt[2] = DoShow3;
   ShowIt[3] = DoShow4;
   FillIt[0] = DoFill1;
   FillIt[1] = DoFill2;
   FillIt[2] = DoFill3;
   FillIt[3] = DoFill4;
   LineWidth[0] = CandleLineWidth1;
   LineWidth[1] = CandleLineWidth2;
   LineWidth[2] = CandleLineWidth3;
   LineWidth[3] = CandleLineWidth4;
   LineStyle[0] = CandleLineStyle1;
   LineStyle[1] = CandleLineStyle2;
   LineStyle[2] = CandleLineStyle3;
   LineStyle[3] = CandleLineStyle4;
   CandleClr[0] = CandleColor1;
   CandleClr[1] = CandleColor2;
   CandleClr[2] = CandleColor3;
   CandleClr[3] = CandleColor4;
   CCBars[0] = CandleBars;
   CCBars[1] = 2 * CandleBars;
   CCBars[2] = 3 * CandleBars;
   CCBars[3] = 4 * CandleBars;
   
   LastBar = Time[Bars-1];
   OpenCC[0] = Close[Bars-1];
   CloseCC[0] = Close[Bars-1];
   OpenCC[1] = Close[Bars-1];
   CloseCC[1] = Close[Bars-1];
   OpenCC[2] = Close[Bars-1];
   CloseCC[2] = Close[Bars-1];
   OpenCC[3] = Close[Bars-1];
   CloseCC[3] = Close[Bars-1];
   
   return(0);
      
} // init()

//----------------------------------------------------------------------------

void ObDeleteObjectsByPrefix(string Prefix)
{
   int L = StringLen(Prefix);
   int i = 0; 
   while(i < ObjectsTotal())
     {
       string ObjName = ObjectName(i);
       if(StringSubstr(ObjName, 0, L) != Prefix) 
         { 
           i++; 
           continue;
         }
       ObjectDelete(ObjName);
     }
}
    
//----------------------------------------------------------------------------
int deinit()
{
       
   ObDeleteObjectsByPrefix(TAG);  
   
   return(0);
   
} // deinit();

//----------------------------------------------------------------------------

int start() 
{
   int limit;
   int i;
   string n;

   limit = iBarShift(Symbol(), 0, LastBar) - 1;

   for (i=limit; i>0; i--)
   {
      CheckCC(i, 0);
      
   } // cycle through all bars 

   return(0);
  
} // start()

//----------------------------------------------------------------------------

void DoPaint(string thetag, int shift, int lev, double theopen, double theclose)
{
   color PaintColor;
   
   if (ShowIt[lev])
      PaintColor = CandleClr[lev];
   else
      PaintColor = CLR_NONE;
   
   ObjectCreate(thetag, OBJ_RECTANGLE, 0, 0, 0);
   ObjectSet(thetag, OBJPROP_BACK, FillIt[lev]);
   ObjectSet(thetag, OBJPROP_WIDTH, LineWidth[lev]);
   ObjectSet(thetag, OBJPROP_STYLE, LineStyle[lev]);
   ObjectSet(thetag, OBJPROP_TIME1, Time[shift+CCBars[lev]-1]);
   ObjectSet(thetag, OBJPROP_PRICE1, theopen);
   ObjectSet(thetag, OBJPROP_TIME2, Time[shift]);
   ObjectSet(thetag, OBJPROP_PRICE2, theclose);
   ObjectSet(thetag, OBJPROP_COLOR, PaintColor);
   
} // DoPaint()

//----------------------------------------------------------------------------

void CheckCC(int shift, int lev)
{
   int j, k, temp, temp2;
   string n;
   
   if (lev == TotLevs)
      return;
      
   if (((lev == 0) && (Close[shift] > Close[shift+LookBack])) || (CloseCC[lev-1] > OpenCC[lev-1]))
   {
      if (CountUp[lev] == (CandleBars - 1))
      {
         CustomBars[lev]++;
         n = TAG + TagIt[lev] + CustomBars[lev];
         OpenCC[lev] = CloseCC[lev];
         if (lev == 0)
            CloseCC[lev] = Close[shift];
         else
            CloseCC[lev] = CloseCC[lev-1];

         DoPaint(n, shift, lev, OpenCC[lev], CloseCC[lev]);

         if (lev > 0)
            Anchor[lev] = CustomBars[lev-1];
            
         CheckCC(shift, lev+1);
      } // completed up CC
      
      CountUp[lev]++;
      CountUp[lev] %= CandleBars;
      CountDown[lev] = 0;
      if (lev == 0)
         LastBar = Time[shift];
      
   } // higher bar or CC
   else
      if (((lev == 0) && (Close[shift] < Close[shift+LookBack])) || (CloseCC[lev-1] < OpenCC[lev-1]))
      {
         if (CountDown[lev] == (CandleBars - 1))
         {
            CustomBars[lev]++;
            n = TAG + TagIt[lev] + CustomBars[lev];
            OpenCC[lev] = CloseCC[lev];
            if (lev == 0)
               CloseCC[lev] = Close[shift];
            else
               CloseCC[lev] = CloseCC[lev-1];

            DoPaint(n, shift, lev, OpenCC[lev], CloseCC[lev]);

            if (lev > 0)
               Anchor[lev] = CustomBars[lev-1];
            
            CheckCC(shift, lev+1);
         } // completed down CC
         
         CountDown[lev]++;
         CountDown[lev] %= CandleBars;
         CountUp[lev] = 0;
         if (lev == 0)
            LastBar = Time[shift];
      } // lower bar or CC
      else
      {
         CountUp[lev] = 0;
         CountDown[lev] = 0;
         if (lev==0)
            LastBar = Time[shift];
      } // doji bar or CC
      
  
   if ((lev == 0) && (shift == 1))
   {
      for (k=0; k<TotLevs; k++)
      {
         n = TAG + TagIt[k] + CustomBars[k];
         if (FixLastCC)
            if (k > 0)
            {
               n = TAG + TagIt[k-1] + Anchor[k];
               temp2 = iBarShift(Symbol(), 0, ObjectGet(n, OBJPROP_TIME2));
            } // Anchor to lower level CC
            else
            {
               n = TAG + TagIt[k] + CustomBars[k];
               temp2 = iBarShift(Symbol(), 0, ObjectGet(n, OBJPROP_TIME2));
            } // anchor to price bar
         else
            temp2 = 0;
         for (j=CustomBars[k]; j>0; j--)
         {
            temp = temp2 + CCBars[k] * (CustomBars[k] - j);
            n = TAG + TagIt[k] + j;
            ObjectSet(n, OBJPROP_TIME1, Time[temp+CCBars[k]-1]);
            ObjectSet(n, OBJPROP_TIME2, Time[temp]);
         } // relocate previous CCs
      } // cycle through all levels
   } // only do this when all history is done 

      
} // CheckCC()

//----------------------------------------------------------------------------