//+------------------------------------------------------------------+ 
//|                                          JS_CC_Trend.mq4         | 
//|   Generates Custom Candles in accordance with the principles     |
//|         laid out by MightyOne in the kreslik.com thread          |
//|         entitled "One Is Not a Trend."                           |
//|                                                                  |
//|                                                                  |
//|   Version 1.1 (6 Feb 12) --- Allows configuration of CC size     |
//|         and LookBack value.  Altering LookBack allows one to     |
//|         imply a fixed CC for the input value.  This enables      |
//|         use with charts like Point and Figure, which would       |
//|         not work with the original settings.                     |
//|         Setting CandleBars to 2 and LookBack to 1 gives          |
//|            identical results to the original version.            |
//|                                                                  |
//|                                                                  |
//|   Version 1.2 (14 Feb 12) --- Made changes to make future        |
//|         maintentance easier.  Also, previous versions had a      |
//|         where setting DoShow for a lower level to false would    |
//|         cause higher levels to also go blank.  This has been     |
//|         fixed.
//+------------------------------------------------------------------+ 

#property copyright "J Scarlett"

#property indicator_chart_window

extern int CandleBars = 2;
extern int LookBack = 1;
extern bool DoShow1 = true;
extern bool DoShow2 = true;
extern bool DoShow3a = false;
extern bool DoShow3b = false;
extern bool DoFill1 = false;
extern bool DoFill2 = true;
extern bool DoFill3a = false;
extern bool DoFill3b = false;
extern int CandleLineWidth1 = 1;
extern int CandleLineStyle1 = 0;
extern int CandleLineWidth2 = 1;
extern int CandleLineStyle2 = 0;
extern int CandleLineWidth3 = 3;
extern int CandleLineStyle3 = 0;
extern color CandleColor1 = Black;
extern color CandleColor2 = Red;
extern color CandleColor3a = PowderBlue;
extern color CandleColor3b = GreenYellow;

// ----globals
datetime LastBar;
int Count1Up = 0, Count1Down = 0;
int Count2Up = 0, Count2Down = 0;
int Count3a = 1, Count3b = 0;
int CustomBars1 = 0, CustomBars2 = 0, CustomBars3a = 0, CustomBars3b = 0;

int digits;
double points;

double Open1, Close1, Open2, Close2, Open3a, Close3a, Open3b, Close3b;

bool FillIt[4];
int LineWidth[4], LineStyle[4], CCBars[4];
color CandleClr[4];

string TAG = "JS_CC_Trend";

//----------------------------------------------------------------------------

int init() 
{
   digits = Digits;
   points = Point; 
   
   deinit();

   if(digits == 5 || digits == 3)
   {
      digits--;
      points *= 10;
   }
   
   FillIt[0] = DoFill1;
   FillIt[1] = DoFill2;
   FillIt[2] = DoFill3a;
   FillIt[3] = DoFill3b;
   LineWidth[0] = CandleLineWidth1;
   LineWidth[1] = CandleLineWidth2;
   LineWidth[2] = CandleLineWidth3;
   LineWidth[3] = CandleLineWidth3;
   LineStyle[0] = CandleLineStyle1;
   LineStyle[1] = CandleLineStyle2;
   LineStyle[2] = CandleLineStyle3;
   LineStyle[3] = CandleLineStyle3;
   CandleClr[0] = CandleColor1;
   CandleClr[1] = CandleColor2;
   CandleClr[2] = CandleColor3a;
   CandleClr[3] = CandleColor3b;
   CCBars[0] = CandleBars;
   CCBars[1] = 2 * CandleBars;
   CCBars[2] = 3 * CandleBars;
   CCBars[3] = 3 * CandleBars;
   
   LastBar = Time[Bars-1];
   Open1 = Close[Bars-1];
   Close1 = Close[Bars-1];
   Open2 = Close[Bars-1];
   Close2 = Close[Bars-1];
   Open3a = Close[Bars-1];
   Close3a = Close[Bars-1];
   Open3b = Close[Bars-1];
   Close3b = Close[Bars-1];
   
   return(0);
      
} // init()

//----------------------------------------------------------------------------

void ObDeleteObjectsByPrefix(string Prefix)
{
   int L = StringLen(Prefix);
   int i = 0; 
   while(i < ObjectsTotal())
     {
       string ObjName = ObjectName(i);
       if(StringSubstr(ObjName, 0, L) != Prefix) 
         { 
           i++; 
           continue;
         }
       ObjectDelete(ObjName);
     }
}
    
//----------------------------------------------------------------------------
int deinit()
{
       
   ObDeleteObjectsByPrefix(TAG);  
   
   return(0);
   
} // deinit();

//----------------------------------------------------------------------------

int start() 
{
   int limit;
   int i;
   string n;

   limit = iBarShift(Symbol(), 0, LastBar) - 1;

   for (i=limit; i>0; i--)
   {
      if (Close[i] > Close[i+LookBack])
      {
         if (Count1Up == (CandleBars - 1))
         {
            CustomBars1++;
            n = TAG + " Lev1 " + CustomBars1;
            Open1 = Close1;
            Close1 = Close[i];
            if (DoShow1)
               DoPaint(n, i, 0, Open1, Close1);
            
            if (Close1 > Open1)
            {
               if (Count2Up == (CandleBars-1))
               {
                  CustomBars2++;
                  n = TAG + " Lev2 " + CustomBars2;
                  Open2 = Close2;
                  Close2 = Close[i];
                  if (DoShow2)
                     DoPaint(n, i, 1, Open2, Close2);
                  
                  if (Count3a == (CandleBars-1))
                  {
                     CustomBars3a++;
                     n = TAG + " Lev3a " + CustomBars3a;
                     Open3a = Close3a;
                     Close3a = Close[i];
                     if (DoShow3a)
                        DoPaint(n, i, 2, Open3a, Close3a);
                  } // Complete level 3a bar 
                  
                  if (Count3b == (CandleBars-1))
                  {
                     CustomBars3b++;
                     n = TAG + " Lev3b " + CustomBars3b;
                     Open3b = Close3b;
                     Close3b = Close[i];
                     if (DoShow3b)
                        DoPaint(n, i, 3, Open3b, Close3b);
                  } // Complete level 3b bar
                  
                  Count3a++;
                  Count3b++;
                  Count3a %= CandleBars;
                  Count3b %= CandleBars;
               } // Complete level 2 up bar;
            
               Count2Up++;
               Count2Up %= CandleBars;
               Count2Down = 0;
            } // increment for level 2 up
            else
               if (Close1 < Open1)
              {
                  if (Count2Down == (CandleBars-1))
                  {
                     CustomBars2++;
                     n = TAG + " Lev2 " + CustomBars2;
                     Open2 = Close2;
                     Close2 = Close[i];
                     if (DoShow2)
                        DoPaint(n, i, 1, Open2, Close2);

                     if (Count3a == (CandleBars-1))
                     {
                        CustomBars3a++;
                        n = TAG + " Lev3a " + CustomBars3a;
                        Open3a = Close3a;
                        Close3a = Close[i];
                        if (DoShow3a)
                           DoPaint(n, i, 2, Open3a, Close3a);
                     } // Complete level 3a bar 
                  
                     if (Count3b == (CandleBars-1))
                     {
                        CustomBars3b++;
                        n = TAG + " Lev3b " + CustomBars3b;
                        Open3b = Close3b;
                        Close3b = Close[i];
                        if (DoShow3b)
                           DoPaint(n, i, 3, Open3b, Close3b);
                     } // Complete level 3b bar
                  
                     Count3a++;
                     Count3b++;
                     Count3a %= CandleBars;
                     Count3b %= CandleBars;
                  } // Complete level 2 up bar;
            
                  Count2Down++;
                  Count2Down %= CandleBars;
                  Count2Up = 0;
               } // increment for level 2 down
               else
               {
                  Count2Up = 0;
                  Count2Down = 0;
               } // this CC was a doji - zero out level 2 counters
               
         } // Complete level 1 bar 
      
         Count1Up++;
         Count1Up %= CandleBars;
         Count1Down = 0;
         LastBar = Time[i];
      } // higher close on this level 1 bar 
      else
         if (Close[i] < Close[i+LookBack])
         {
            if (Count1Down == (CandleBars - 1))
            {
               CustomBars1++;
               n = TAG + " Lev1 " + CustomBars1;
               Open1 = Close1;
               Close1 = Close[i];
               if (DoShow1)
                  DoPaint(n, i, 0, Open1, Close1);
               
               if (Close1 > Open1)
               {
                  if (Count2Up == (CandleBars-1))
                  {
                     CustomBars2++;
                     n = TAG + " Lev2 " + CustomBars2;
                     Open2 = Close2;
                     Close2 = Close[i];
                     if (DoShow2)
                        DoPaint(n, i, 1, Open2, Close2);

                     if (Count3a == (CandleBars-1))
                     {
                        CustomBars3a++;
                        n = TAG + " Lev3a " + CustomBars3a;
                        Open3a = Close3a;
                        Close3a = Close[i];
                        if (DoShow3a)
                           DoPaint(n, i, 2, Open3a, Close3a);
                     } // Complete level 3a bar 
                  
                     if (Count3b == (CandleBars-1))
                     {
                        CustomBars3b++;
                        n = TAG + " Lev3b " + CustomBars3b;
                        Open3b = Close3b;
                        Close3b = Close[i];
                        if (DoShow3b)
                           DoPaint(n, i, 3, Open3b, Close3b);
                     } // Complete level 3b bar
                  
                     Count3a++;
                     Count3b++;
                     Count3a %= CandleBars;
                     Count3b %= CandleBars;
                  } // Complete level 2 up bar;
            
                  Count2Up++;
                  Count2Up %= CandleBars;
                  Count2Down = 0;
               } // increment for level 2 up
               else
                  if (Close1 < Open1)
                  {
                     if (Count2Down == (CandleBars-1))
                     {
                        CustomBars2++;
                        n = TAG + " Lev2 " + CustomBars2;
                        Open2 = Close2;
                        Close2 = Close[i];
                        if (DoShow2)
                           DoPaint(n, i, 1, Open2, Close2);

                        if (Count3a == (CandleBars-1))
                        {
                           CustomBars3a++;
                           n = TAG + " Lev3a " + CustomBars3a;
                           Open3a = Close3a;
                           Close3a = Close[i];
                           if (DoShow3a)
                              DoPaint(n, i, 2, Open3a, Close3a);
                        } // Complete level 3a bar 
                  
                        if (Count3b == (CandleBars-1))
                        {
                           CustomBars3b++;
                           n = TAG + " Lev3b " + CustomBars3b;
                           Open3b = Close3b;
                           Close3b = Close[i];
                           if (DoShow3b)
                              DoPaint(n, i, 3, Open3b, Close3b);
                        } // Complete level 3b bar
                  
                        Count3a++;
                        Count3b++;
                        Count3a %= CandleBars;
                        Count3b %= CandleBars;
                     } // Complete level 2 up bar;
            
                     Count2Down++;
                     Count2Down %= CandleBars;
                     Count2Up = 0;
                  } // increment for level 2 down
                  else
                  {
                     Count2Up = 0;
                     Count2Down = 0;
                  } // this CC was a doji - zero out level 2 counters
                  
            } // complete level 1 bar
                        
            Count1Down++;
            Count1Down %= CandleBars;
            Count1Up = 0;
            LastBar = Time[i];
         } // lower close on this level 1 Bars
         else
         {
            Count1Up = 0;
            Count1Down = 0;
            LastBar = Time[i];
         } // doji - reset level 1 counts
   

   } // cycle through all bars 

   return(0);
  
} // start()

//----------------------------------------------------------------------------

void DoPaint(string thetag, int shift, int lev, double theopen, double theclose)
{
   ObjectCreate(thetag, OBJ_RECTANGLE, 0, 0, 0);
   ObjectSet(thetag, OBJPROP_BACK, FillIt[lev]);
   ObjectSet(thetag, OBJPROP_WIDTH, LineWidth[lev]);
   ObjectSet(thetag, OBJPROP_STYLE, LineStyle[lev]);
   ObjectSet(thetag, OBJPROP_TIME1, Time[shift+CCBars[lev]-1]);
   ObjectSet(thetag, OBJPROP_PRICE1, theopen);
   ObjectSet(thetag, OBJPROP_TIME2, Time[shift]);
   ObjectSet(thetag, OBJPROP_PRICE2, theclose);
   ObjectSet(thetag, OBJPROP_COLOR, CandleClr[lev]);
} // DoPaint()

//----------------------------------------------------------------------------