//+------------------------------------------------------------------+ 
//|                                    JS_CC_Trend_Stacked.mq4       | 
//|                                                                  |
//|   Uses Mighty One's TrendSetter idea as its basis.  Once the     |
//|         custom candles are formed, they are "stacked."  This     |
//|         works in a similar fashion to Point & Figure charts.     |
//|         Consecutive up candles are stacked into one candle       |
//|         until there is a down candle.  Then consecutive down     |
//|         candles are stacked in the same manner, etc.             |
//|      There is provision for the user to choose which TS level    |
//|         is used (up to 4), but only one CC chart is displayed.   |
//|                                                                  |
//+------------------------------------------------------------------+ 

#property copyright "J Scarlett"

#property indicator_chart_window

extern int CandleBars = 2;
extern int LookBack = 1;
extern int CC_Level = 1;
extern bool DoFill = true;
extern int CandleLineWidth = 1;
extern int CandleLineStyle = 0;
extern color CandleColor = Blue;

// ----globals
bool IsUp;
datetime LastBar;
int TotLevs;
int CountUp[4] = {0, 0, 0, 0};
int CountDown[4] = {0, 0, 0, 0};
int CustomBars[4] = {0, 0, 0, 0};
int Anchor[4] = {0, 0, 0, 0};
int CCBars[4];
int StackedBars = 0;
int LastCC;

int digits;
double points;

double OpenCC[4], CloseCC[4];
double StackOpen, StackClose;

bool FillIt;
int LineWidth, LineStyle;

string TAG = "JS_TS_Stacked ";
string TAG1;
string TAG2 = "Raw ";

//----------------------------------------------------------------------------

int init() 
{
   digits = Digits;
   points = Point; 
   
   deinit();

   if(digits == 5 || digits == 3)
   {
      digits--;
      points *= 10;
   }
   
   FillIt = DoFill;
   LineWidth = CandleLineWidth;
   LineStyle = CandleLineStyle;
   CCBars[0] = CandleBars;
   CCBars[1] = CandleBars;
   CCBars[2] = CandleBars;
   CCBars[3] = CandleBars;
   if (CC_Level < 5)
      TotLevs = CC_Level;
   else
      TotLevs = 4;
   TAG1 = TotLevs + " ";
   
   LastBar = Time[Bars-1];
   OpenCC[0] = Close[Bars-1];
   CloseCC[0] = Close[Bars-1];
   OpenCC[1] = Close[Bars-1];
   CloseCC[1] = Close[Bars-1];
   OpenCC[2] = Close[Bars-1];
   CloseCC[2] = Close[Bars-1];
   OpenCC[3] = Close[Bars-1];
   CloseCC[3] = Close[Bars-1];
   
   return(0);
      
} // init()

//----------------------------------------------------------------------------

void ObDeleteObjectsByPrefix(string Prefix)
{
   int L = StringLen(Prefix);
   int i = 0; 
   while(i < ObjectsTotal())
     {
       string ObjName = ObjectName(i);
       if(StringSubstr(ObjName, 0, L) != Prefix) 
         { 
           i++; 
           continue;
         }
       ObjectDelete(ObjName);
     }
}
    
//----------------------------------------------------------------------------
int deinit()
{
       
   ObDeleteObjectsByPrefix(TAG);  
   
   return(0);
   
} // deinit();

//----------------------------------------------------------------------------

int start() 
{
   int limit;
   int i;
   string n;

   limit = iBarShift(Symbol(), 0, LastBar) - 1;

   for (i=limit; i>0; i--)
   {
      CheckCC(i, 0);
      
   } // cycle through all bars 

   return(0);
  
} // start()

//----------------------------------------------------------------------------

void DoPaintRaw(string thetag, int shift, int lev, double theopen, double theclose)
{
   color PaintColor;
   
//   if (ShowIt[lev])
//      PaintColor = CandleClr[lev];
//   else
      PaintColor = CLR_NONE;
   
   ObjectCreate(thetag, OBJ_RECTANGLE, 0, 0, 0);
//   ObjectSet(thetag, OBJPROP_BACK, FillIt[lev]);
   ObjectSet(thetag, OBJPROP_WIDTH, LineWidth);
   ObjectSet(thetag, OBJPROP_STYLE, LineStyle);
   ObjectSet(thetag, OBJPROP_TIME1, Time[shift+CCBars[lev]-1]);
   ObjectSet(thetag, OBJPROP_PRICE1, theopen);
   ObjectSet(thetag, OBJPROP_TIME2, Time[shift]);
   ObjectSet(thetag, OBJPROP_PRICE2, theclose);
   ObjectSet(thetag, OBJPROP_COLOR, PaintColor);
   
} // DoPaint()

//----------------------------------------------------------------------------

void CheckCC(int shift, int lev)
{
   int j, k, temp, temp2;
   string n;
   
   if (lev == TotLevs)
      return;
      
   if (((lev == 0) && (Close[shift] > Close[shift+LookBack])) || (CloseCC[lev-1] > OpenCC[lev-1]))
   {
      if (CountUp[lev] == (CandleBars - 1))
      {
         CustomBars[lev]++;
         n = TAG + TAG2 + CustomBars[lev];
         OpenCC[lev] = CloseCC[lev];
         if (lev == 0)
            CloseCC[lev] = Close[shift];
         else
            CloseCC[lev] = CloseCC[lev-1];

         DoPaintRaw(n, shift, lev, OpenCC[lev], CloseCC[lev]);

         if (lev > 0)
            Anchor[lev] = CustomBars[lev-1];
            
         CheckCC(shift, lev+1);
      } // completed up CC
      
      CountUp[lev]++;
      CountUp[lev] %= CandleBars;
      CountDown[lev] = 0;
      if (lev == 0)
         LastBar = Time[shift];
      
   } // higher bar or CC
   else
      if (((lev == 0) && (Close[shift] < Close[shift+LookBack])) || (CloseCC[lev-1] < OpenCC[lev-1]))
      {
         if (CountDown[lev] == (CandleBars - 1))
         {
            CustomBars[lev]++;
            n = TAG + TAG2 + CustomBars[lev];
            OpenCC[lev] = CloseCC[lev];
            if (lev == 0)
               CloseCC[lev] = Close[shift];
            else
               CloseCC[lev] = CloseCC[lev-1];

            DoPaintRaw(n, shift, lev, OpenCC[lev], CloseCC[lev]);

            if (lev > 0)
               Anchor[lev] = CustomBars[lev-1];
            
            CheckCC(shift, lev+1);
         } // completed down CC
         
         CountDown[lev]++;
         CountDown[lev] %= CandleBars;
         CountUp[lev] = 0;
         if (lev == 0)
            LastBar = Time[shift];
      } // lower bar or CC
      else
      {
         CountUp[lev] = 0;
         CountDown[lev] = 0;
         if (lev==0)
            LastBar = Time[shift];
      } // doji bar or CC
      
  
   if ((lev == 0) && (shift == 1))
   {
      if (LastCC < CustomBars[TotLevs-1])
      {
         DoStack();
         
         for (j=StackedBars; j>0; j--)
         {
            temp2 = CCBars[TotLevs-1];
            temp = temp2 * (StackedBars - j) + 1;
            n = TAG + TAG1 + j;
            ObjectSet(n, OBJPROP_TIME1, Time[temp+temp2-1]);
            ObjectSet(n, OBJPROP_TIME2, Time[temp]);
         } // relocate stacked CCs
      } // new TS CC - update the stacked CC
      
   } // only do this when all history is done 

      
} // CheckCC()

//----------------------------------------------------------------------------

void DoStack()
{
   double temp;
   string n1, n2;
   
   if (StackedBars == 0)
   {
      n1 = TAG + TAG2 + 1;
      n2 = TAG + TAG1 + 1;
      ObjectCreate(n2, OBJ_RECTANGLE, 0, 0, 0);
      ObjectSet(n2, OBJPROP_BACK, DoFill);
      ObjectSet(n2, OBJPROP_WIDTH, LineWidth);
      ObjectSet(n2, OBJPROP_STYLE, LineStyle);
      ObjectSet(n2, OBJPROP_TIME1, ObjectGet(n1, OBJPROP_TIME1));
      ObjectSet(n2, OBJPROP_TIME2, ObjectGet(n1, OBJPROP_TIME2));
      ObjectSet(n2, OBJPROP_COLOR, CandleColor);
      
      StackOpen = ObjectGet(n1, OBJPROP_PRICE1);
      StackClose = ObjectGet(n1, OBJPROP_PRICE2);
      if (StackClose >= StackOpen)
         IsUp = true;
      else
         IsUp = false;
         
      ObjectSet(n2, OBJPROP_PRICE1, StackOpen);
      ObjectSet(n2, OBJPROP_PRICE2, StackClose);
      
      StackedBars++;
      LastCC++;
   } // no stacked bars created yet

   while (LastCC < CustomBars[TotLevs-1])
   {
      LastCC++;
      if (IsUp)
      {
         n1 = TAG + TAG2 + LastCC;
         temp = ObjectGet(n1, OBJPROP_PRICE2);
         if (temp >= StackClose)
         {
            StackClose = temp;
            n2 = TAG + TAG1 + StackedBars;
            ObjectSet(n2, OBJPROP_PRICE2, StackClose);
         } // continue up stack CC
         else
         {
            StackedBars++;
            IsUp = false;
            n2 = TAG + TAG1 + StackedBars;
            
            StackOpen = ObjectGet(n1, OBJPROP_PRICE1);
            StackClose = temp;

            ObjectCreate(n2, OBJ_RECTANGLE, 0, 0, 0);
            ObjectSet(n2, OBJPROP_BACK, DoFill);
            ObjectSet(n2, OBJPROP_WIDTH, LineWidth);
            ObjectSet(n2, OBJPROP_STYLE, LineStyle);
            ObjectSet(n2, OBJPROP_TIME1, ObjectGet(n1, OBJPROP_TIME1));
            ObjectSet(n2, OBJPROP_TIME2, ObjectGet(n1, OBJPROP_TIME2));
            ObjectSet(n2, OBJPROP_PRICE1, StackOpen);
            ObjectSet(n2, OBJPROP_PRICE2, StackClose);
            ObjectSet(n2, OBJPROP_COLOR, CandleColor);
         } // begin a down stack CC
         
      } // current stacked CC is up
      else
      {
         n1 = TAG + TAG2 + LastCC;
         temp = ObjectGet(n1, OBJPROP_PRICE2);
         if (temp <= StackClose)
         {
            StackClose = temp;
            n2 = TAG + TAG1 + StackedBars;
            ObjectSet(n2, OBJPROP_PRICE2, StackClose);
         } // continue up stack CC
         else
         {
            StackedBars++;
            IsUp = true;
            n2 = TAG + TAG1 + StackedBars;
            
            StackOpen = ObjectGet(n1, OBJPROP_PRICE1);
            StackClose = temp;

            ObjectCreate(n2, OBJ_RECTANGLE, 0, 0, 0);
            ObjectSet(n2, OBJPROP_BACK, DoFill);
            ObjectSet(n2, OBJPROP_WIDTH, LineWidth);
            ObjectSet(n2, OBJPROP_STYLE, LineStyle);
            ObjectSet(n2, OBJPROP_TIME1, ObjectGet(n1, OBJPROP_TIME1));
            ObjectSet(n2, OBJPROP_TIME2, ObjectGet(n1, OBJPROP_TIME2));
            ObjectSet(n2, OBJPROP_PRICE1, StackOpen);
            ObjectSet(n2, OBJPROP_PRICE2, StackClose);
            ObjectSet(n2, OBJPROP_COLOR, CandleColor);
         } // begin a down stack CC
         
      } // current stacked CC is down
   } // cycle through any new CC
   
   return;
   
} // DoStack()

//----------------------------------------------------------------------------